--------------------------------------------------------------------------------
-- Dictionary Attack
-- version 1.1
--------------------------------------------------------------------------------

-- Setup Tables
if kifferopa==nil then kifferopa={} end
kifferopa.dictionaryattack={}
kifferopa.dictionaryattack.cluster={}
kifferopa.dictionaryattack.plane={}

-- Load & Prepare Ressources
kifferopa.dictionaryattack.gfx_wpn=loadgfx("weapons/rc.bmp")					-- Weapon Image
setmidhandle(kifferopa.dictionaryattack.gfx_wpn)
kifferopa.dictionaryattack.gfx_icon=loadgfx("weapons/dictionaryattack_icon.png")		-- Weapon Icon
setmidhandle(kifferopa.dictionaryattack.gfx_icon)
kifferopa.dictionaryattack.gfx_pro=loadgfx("weapons/dictionary.png")				-- Projectile Image
setmidhandle(kifferopa.dictionaryattack.gfx_pro)
kifferopa.dictionaryattack.sfx_attack=loadsfx("airstrike.ogg")					-- Attack Sound
kifferopa.dictionaryattack.gfx_plane=loadgfx("weapons/airstrike_airplane.png")			-- airplane image
setmidhandle(kifferopa.dictionaryattack.gfx_plane)

--------------------------------------------------------------------------------
-- Weapon: Airstrike
--------------------------------------------------------------------------------

kifferopa.dictionaryattack.id=addweapon("kifferopa.dictionaryattack","Dictionary Attack",kifferopa.dictionaryattack.gfx_icon,0,2)	-- Add Weapon (0 uses, first in round 2)

function kifferopa.dictionaryattack.draw()													-- Draw
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	drawinhand(kifferopa.dictionaryattack.gfx_wpn,7,0)
	-- HUD Positioning
	if weapon_shots==0 then
		hudpositioning(pos_invisible)
	end
end

function kifferopa.dictionaryattack.attack(attack)									-- Attack
	if (weapon_shots<=0) and (weapon_position==1) then
		-- No more weapon switching!
		useweapon(0)
		playsound(kifferopa.dictionaryattack.sfx_attack)
		weapon_shots=weapon_shots+1
		id=createprojectile(kifferopa.dictionaryattack.plane.id)
		projectiles[id]={}
		if(getplayerdirection(0)==1) then
			projectiles[id].x=-1000
			projectiles[id].sx=40
		else
			projectiles[id].x=getmapwidth()+1000
			projectiles[id].sx=-40
		end
		projectiles[id].y=-512
		projectiles[id].dropx=weapon_x
		projectiles[id].throwindex=-2
		weapon_mode=0
		endturn()
	end
end

--------------------------------------------------------------------------------
-- Projectile: Plane
--------------------------------------------------------------------------------
kifferopa.dictionaryattack.plane.id=addprojectile("kifferopa.dictionaryattack.plane")
function kifferopa.dictionaryattack.plane.draw(id)
	local dir
	-- direction
	if(projectiles[id].sx>0) then
		dir=1
	else
		dir=-1
	end
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(dir,1)
	setrotation(0)
	drawimage(kifferopa.dictionaryattack.gfx_plane,projectiles[id].x,projectiles[id].y)
	if(weapon_mode==0) then
		scroll(projectiles[id].x,projectiles[id].y)
	end
end

function kifferopa.dictionaryattack.plane.update(id)
	local dir
	if(projectiles[id].sx>0) then
		dir=1
	else
		dir=-1
	end
	-- Particles
	particle(p_smoke, projectiles[id].x-55*dir, projectiles[id].y+11)
	particlespeed(math.random(-2,2)*0.1, math.random(-2,2)*0.1)
	particlefadealpha(0.01)
	
	particle(p_lightpuff,projectiles[id].x-55*dir, projectiles[id].y+11)
	particlefadealpha(0.04)

	-- move
	projectiles[id].x=projectiles[id].x+projectiles[id].sx

	if weapon_mode==0 then
		local dropx=projectiles[id].throwindex*50*dir
		if(dir==1 and projectiles[id].x>=projectiles[id].dropx+dropx) or (dir==-1 and projectiles[id].x<=projectiles[id].dropx+dropx) then

			-- Attack
			projectiles[id].throwindex=projectiles[id].throwindex+1
			pid=createprojectile(kifferopa.dictionaryattack.cluster.id)
			projectiles[pid]={}
			projectiles[pid].x=projectiles[id].dropx+dropx
			projectiles[pid].y=-500
			projectiles[pid].sx=1.5*dir
			projectiles[pid].sy=3.0
			projectiles[pid].collisions=0
			if projectiles[id].throwindex==0 then
				projectiles[pid].follow=true
			else
				projectiles[pid].follow=false
			end
			if(projectiles[id].throwindex>2) then
				weapon_mode=1
			end
		end
	end
		
	-- remove plane
	if(dir==1 and projectiles[id].x>projectiles[id].dropx+500 and projectiles[id].x>(getmapwidth()+500)) or (dir==-1 and projectiles[id].x<projectiles[id].dropx-500 and projectiles[id].x<-500) then
		freeprojectile(id)
	end
end

--------------------------------------------------------------------------------
-- Projectile: Cluster
--------------------------------------------------------------------------------

kifferopa.dictionaryattack.cluster.id=addprojectile("kifferopa.dictionaryattack.cluster")	-- Add Projectile

function kifferopa.dictionaryattack.cluster.draw(id)							-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(kifferopa.dictionaryattack.gfx_pro,projectiles[id].x,projectiles[id].y)
end

function kifferopa.dictionaryattack.cluster.update(id)							-- Update
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))

	-- Wind + Gravity influence on speed
	projectiles[id].sx=projectiles[id].sx+getwind()*0.05
	projectiles[id].sy=projectiles[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		-- Move X
		projectiles[id].x=projectiles[id].x+msubx
		if collision(kifferopa.dictionaryattack.gfx_pro, projectiles[id].x, projectiles[id].y, 1, 1)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				-- Cause damage
				arealdamage(projectiles[id].x,projectiles[id].y,70,25)
				-- Destroy terrain
				terrainexplosion(projectiles[id].x,projectiles[id].y,20,1)
				-- Crater
				grey=math.random(0,40)
				terrainalphaimage(gfx_crater75,projectiles[id].x,projectiles[id].y,math.random(5,7)*0.1,grey,grey,grey)

				projectiles[id].x=projectiles[id].x-msubx
				projectiles[id].sx=-projectiles[id].sx*0.9
				msubx=-msubx*0.9
				projectiles[id].collisions = projectiles[id].collisions + 1	
				
				if(projectiles[id].collisions >= 4) then
					freeprojectile(id)
				end
			end
		else
			projectiles[id].ignore=0
		end
		-- Move Y
		projectiles[id].y=projectiles[id].y+msuby
		if collision(kifferopa.dictionaryattack.gfx_pro, projectiles[id].x,projectiles[id].y,1,1)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				-- Cause damage
				arealdamage(projectiles[id].x,projectiles[id].y,70,25)
				-- Destroy terrain
				terrainexplosion(projectiles[id].x,projectiles[id].y,20,1)
				-- Crater
				grey=math.random(0,40)
				terrainalphaimage(gfx_crater75,projectiles[id].x,projectiles[id].y,math.random(5,7)*0.1,grey,grey,grey)

				projectiles[id].y=projectiles[id].y-msuby
				projectiles[id].sy=-projectiles[id].sy*0.5
				msuby=-msuby*0.5
				projectiles[id].collisions = projectiles[id].collisions + 1

				if(projectiles[id].collisions >= 4) then
					freeprojectile(id)
				end
			end
		else
			projectiles[id].ignore=0
		end	
		
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- Free projectile
			freeprojectile(id)
			break
		end
	end
	-- Scroll to projectile
	if projectiles[id].follow==true then
		scroll(projectiles[id].x,projectiles[id].y)
	end
end
